from sklearn.base import BaseEstimator, TransformerMixin


class ExtractLetterTransformer(BaseEstimator, TransformerMixin):
    # Extract fist letter of variable

    def __init__(self, variables):
        if not isinstance(variables, list):
            raise ValueError("series must be a list")

        self.variables = variables

    def fit(self, X, y=None):
        return self

    def transform(self, X):
        X = X.copy()
        for var in self.variables:
            X[var] = X[var].str[0]
        return X



from pathlib import Path
from typing import Optional, Sequence

from pydantic import BaseModel
from strictyaml import YAML, load

import classification_model

# Project directories
PACKAGE_ROOT = Path(classification_model.__file__).resolve().parent
CONFIG_FILE_PATH = PACKAGE_ROOT / "config.yml"
TRAINED_MODELS_DIR = PACKAGE_ROOT / "trained_models"


class AppConfig(BaseModel):
    package_name: str
    pipeline_save_file: str


class ModelConfig(BaseModel):
    target: str
    numerical_variables: Sequence[str]
    categorical_variables: Sequence[str]
    cabin: Sequence[str]
    features: Sequence[str]
    unused_fields: Sequence[str]
    path_dataset: str
    test_size: float
    seed: int


class Config(BaseModel):
    app_config: AppConfig
    model_config: ModelConfig


def find_config_file() -> Path:
    """Locate the config file"""
    if CONFIG_FILE_PATH.is_file():
        return CONFIG_FILE_PATH
    else:
        raise Exception(f"Config file not found at {CONFIG_FILE_PATH!r}")


def load_config_from_yaml(config_path: Optional[Path] = None) -> YAML:
    if not config_path:
        config_path = find_config_file()

    with open(config_path) as file:
        config = load(file.read()).data
        return config

    raise OSError(f"Did not find config file at path {config_path}")


def create_and_validate_config(
    parsed_config: YAML = None,
) -> Config:
    if parsed_config is None:
        parsed_config = load_config_from_yaml()

    config = Config(
        app_config=AppConfig(**parsed_config), model_config=ModelConfig(**parsed_config)
    )
    return config


config = create_and_validate_config()



import pandas as pd

from classification_model.config.core import config
from classification_model.preprocessing.features import ExtractLetterTransformer


def test_temporal_variable_transformes(sample_input_data):

    X_test, y_test = sample_input_data

    transformer = ExtractLetterTransformer(variables=config.model_config.cabin)
    X_test = transformer.transform(X_test)

    assert X_test["cabin"].iat[0] == "?"

    assert X_test["sex"].iat[1] == "female"
    assert X_test.shape[0] == len(y_test)

    assert isinstance(X_test, pd.DataFrame)
    assert isinstance(y_test, pd.Series)



from classification_model.preprocessing.data_manager import get_title


def test_get_title_with_array(get_titles):
    titles = [get_title(title) for title in get_titles]
    assert titles == ["Mrs", "Mr", "Miss", "Other", "Master"]



from typing import Tuple

import numpy as np
import pandas as pd

from classification_model.predict import make_prediction


def test_predict(sample_input_data: Tuple[pd.DataFrame, pd.Series]):
    X_test, _ = sample_input_data

    results = make_prediction(input_data=X_test)
    predictions = results.get("predictions")

    assert results.get("errors") is None
    assert isinstance(predictions[0], np.int64)
    assert len(predictions) == X_test.shape[0]



import re
from typing import List

import joblib
import numpy as np
import pandas as pd
from sklearn.pipeline import Pipeline

from classification_model import __version__ as _version
from classification_model.config.core import TRAINED_MODELS_DIR, config


def load_dataset() -> pd.DataFrame:
    data = pd.read_csv(config.model_config.path_dataset)
    return pre_pipeline_preparation(data)


def _load_raw_dataset() -> pd.DataFrame:
    return pd.read_csv(config.model_config.path_dataset)


def pre_pipeline_preparation(dataframe: pd.DataFrame) -> pd.DataFrame:
    """Prepipeline preparation

    Args:
        dataframe (pd.DataFrame): Pandas dataframe

    Returns:
        pd.DataFrame: Dataframe preprocessed
    """
    data = dataframe.replace("?", np.nan)

    data["title"] = data["name"].apply(get_title)

    data["fare"] = data["fare"].astype("float")
    data["age"] = data["age"].astype("float")
    data.drop(labels=config.model_config.unused_fields, axis=1, inplace=True)

    return data


def get_title(passenger: str) -> str:
    line = passenger
    if re.search("Mrs", line):
        return "Mrs"
    elif re.search("Mr", line):
        return "Mr"
    elif re.search("Miss", line):
        return "Miss"
    elif re.search("Master", line):
        return "Master"
    else:
        return "Other"


def save_pipeline(pipeline: Pipeline) -> None:
    pipeline_name = f"{config.app_config.pipeline_save_file}{_version}.joblib"

    remove_old_pipelines(files_to_keep=[pipeline_name])

    path = TRAINED_MODELS_DIR / pipeline_name

    joblib.dump(pipeline, path)


def remove_old_pipelines(files_to_keep: List[str]) -> None:
    do_not_delete = files_to_keep + ["__init__.py"]
    for file in TRAINED_MODELS_DIR.iterdir():
        if file not in do_not_delete:
            file.unlink()


def load_pipeline(file_name: str) -> Pipeline:
    path = f"{TRAINED_MODELS_DIR}/{file_name}"
    return joblib.load(path)
